@@ -21,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::ASF;   # for GetGUID()
 
-$VERSION = '1.34';
+$VERSION = '1.38';
 
 sub ProcessFPX($$);
 sub ProcessFPXR($$$);
@@ -125,7 +125,7 @@ my @dirEntryType = qw(INVALID STORAGE STREAM LOCKBYTES PROPERTY ROOT);
 # list of code pages used by Microsoft
 # (ref http://msdn.microsoft.com/en-us/library/dd317756(VS.85).aspx)
 my %codePage = (
-    037 => 'IBM EBCDIC US-Canada',
+     37 => 'IBM EBCDIC US-Canada',
     437 => 'DOS United States',
     500 => 'IBM EBCDIC International',
     708 => 'Arabic (ASMO 708)',
@@ -313,7 +313,8 @@ my %fpxFileType = (
         from DOC, PPT, XLS (Microsoft Word, PowerPoint and Excel) documents, VSD
         (Microsoft Visio) drawings, and FLA (Macromedia/Adobe Flash project) files
         since these are based on the same file format as FlashPix (the Windows
-        Compound Binary File format).  See
+        Compound Binary File format).  Note that ExifTool identifies any
+        unrecognized Windows Compound Binary file as a FlashPix (FPX) file.  See
         L<http://graphcomp.com/info/specs/livepicture/fpx.pdf> for the FlashPix
         specification.
     },
@@ -1164,10 +1165,7 @@ my %fpxFileType = (
     9.2 => {
         Name => 'Word97',
         Mask => 0x0010,
-        PrintConv => {
-            0x0000 => 'No',
-            0x0010 => 'Yes',
-        },
+        PrintConv => { 0 => 'No', 1 => 'Yes' },
     },
 );
 
@@ -1178,16 +1176,17 @@ my %fpxFileType = (
     VARS => { NO_ID => 1 },
     CommentBy => {
         Groups => { 2 => 'Author' },
-        Notes => 'enable Duplicates option to extract all entries',
+        Notes => 'enable L<Duplicates|../ExifTool.html#Duplicates> option to extract all entries',
     },
     LastSavedBy => {
         Groups => { 2 => 'Author' },
-        Notes => 'enable Duplicates option to extract history of up to 10 entries',
+        Notes => 'enable L<Duplicates|../ExifTool.html#Duplicates> option to extract history of up to 10 entries',
     },
     DOP => { SubDirectory => { TagTable => 'Image::ExifTool::FlashPix::DOP' } },
     ModifyDate => {
         Groups => { 2 => 'Time' },
         Format => 'int64u',
+        Priority => 0,
         RawConv => q{
             $val = $val * 1e-7 - 11644473600;   # convert to seconds since 1970
             return $val > 0 ? $val : undef;
@@ -1441,7 +1440,7 @@ sub ReadFPXValue($$$$$;$$)
                     my $charset = $Image::ExifTool::charsetName{"cp$codePage"};
                     if ($charset) {
                         $val = $et->Decode($val, $charset);
-                    } elsif ($codePage eq 1200) {   # UTF-16, little endian
+                    } elsif ($codePage == 1200) {   # UTF-16, little endian
                         $val = $et->Decode($val, 'UCS2', 'II');
                     }
                 }
@@ -1520,9 +1519,15 @@ sub ProcessContents($$$)
 sub ProcessWordDocument($$$)
 {
     my ($et, $dirInfo, $tagTablePtr) = @_;
-    $et->ProcessBinaryData($dirInfo, $tagTablePtr);
     my $dataPt = $$dirInfo{DataPt} or return 0;
     my $dirLen = length $$dataPt;
+    # validate the FIB signature
+    unless ($dirLen > 2 and Get16u($dataPt,0) == 0xa5ec) {
+        $et->WarnOnce('Invalid FIB signature', 1);
+        return 0;
+    }
+    $et->ProcessBinaryData($dirInfo, $tagTablePtr); # process FIB
+    # continue parsing the WordDocument stream until we find the FibRgFcLcb
     my $pos = 32;
     return 0 if $pos + 2 > $dirLen;
     my $n = Get16u($dataPt, $pos);  # read csw
@@ -1573,14 +1578,16 @@ sub ProcessDocumentTable($)
         my $key = 'TableOffsets' . ($i ? " ($i)" : '');
         my $offsets = $$value{$key};
         last unless defined $offsets;
-        my $doc = $$extra{$key}{G3} if $$extra{$key};
+        my $doc;
+        $doc = $$extra{$key}{G3} if $$extra{$key};
         $doc = '' unless $doc;
         # get DocFlags for this sub-document
         my ($docFlags, $docTable);
         for ($j=0; ; ++$j) {
             my $key = 'DocFlags' . ($j ? " ($j)" : '');
             last unless defined $$value{$key};
-            my $tmp = $$extra{$key}{G3} if $$extra{$key};
+            my $tmp;
+            $tmp = $$extra{$key}{G3} if $$extra{$key};
             $tmp = '' unless $tmp;
             if ($tmp eq $doc) {
                 $docFlags = $$value{$key};
@@ -1593,7 +1600,8 @@ sub ProcessDocumentTable($)
         for ($j=0; ; ++$j) {
             my $key = $tag . ($j ? " ($j)" : '');
             last unless defined $$value{$key};
-            my $tmp = $$extra{$key}{G3} if $$extra{$key};
+            my $tmp;
+            $tmp = $$extra{$key}{G3} if $$extra{$key};
             $tmp = '' unless $tmp;
             if ($tmp eq $doc) {
                 $docTable = \$$value{$key};
@@ -1991,7 +1999,7 @@ sub ProcessFPXR($$$)
             $et->Warn("Unlisted FPXR segment (index $index)") if $index != 255;
         }
 
-    } elsif ($type ne 3) {  # not a "Reserved" segment
+    } elsif ($type != 3) {  # not a "Reserved" segment
 
         $et->Warn("Unknown FPXR segment (type $type)");
 
@@ -2362,7 +2370,7 @@ JPEG images.
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
