@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.02';
+$VERSION = '1.03';
 
 sub ProcessUserProps($$$);
 
@@ -154,9 +154,11 @@ sub UnescapeRTF($$$)
         return $val;
     }
     # CR/LF is signficant if it terminates a control sequence (so change these to a space)
-    $val =~ s/(^|[^\\])((?:\\\\)*)(\\[a-zA-Z]+(?:-?\d+)?)[\n\r]/$1$2$3 /g;
+    # (was $val =~ s/(^|[^\\])((?:\\\\)*)(\\[a-zA-Z]+(?:-?\d+)?)[\n\r]/$1$2$3 /g;)
+    $val =~ s/\\(?:([a-zA-Z]+(?:-?\d+)?)[\n\r]|(.))/'\\'.($1 ? "$1 " : $2)/sge;
     # protect the newline control sequence by converting to a \par command
-    $val =~ s/(^|[^\\])((?:\\\\)*)(\\[\n\r])/$1$2\\par /g;
+    # (was $val =~ s/(^|[^\\])((?:\\\\)*)(\\[\n\r])/$1$2\\par /g;)
+    $val =~ s/(\\[\n\r])|(\\.)/$2 || '\\par '/sge;
     # all other CR/LF's are ignored (so delete them)
     $val =~ tr/\n\r//d;
 
@@ -223,14 +225,14 @@ sub ProcessRTF($$)
     my ($et, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my ($buff, $buf2, $cs);
-    
+
     return 0 unless $raf->Read($buff, 64) and $raf->Seek(0,0);
     return 0 unless $buff =~ /^[\n\r]*\{[\n\r]*\\rtf[^a-zA-Z]/;
     $et->SetFileType();
 #
 # determine the RTF character set
 #
-    if ($buff=~ /\\ansicpg(\d*)/) { 
+    if ($buff=~ /\\ansicpg(\d*)/) {
         $cs = "cp$1";
     } elsif ($buff=~ /\\(ansi|mac|pc|pca)[^a-zA-Z]/) {
         my %trans = (
@@ -359,7 +361,7 @@ information from RTF (Rich Text Format) documents.
 
 =head1 AUTHOR
 
-Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
