@@ -11,8 +11,9 @@ package Image::ExifTool::SonyIDC;
 use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
+use Image::ExifTool::Exif;
 
-$VERSION = '1.02';
+$VERSION = '1.08';
 
 # Sony IDC tags (ref PH)
 %Image::ExifTool::SonyIDC::Main = (
@@ -91,7 +92,12 @@ $VERSION = '1.02';
     0x8015 => { Name => 'ColorCorrection',      Writable => 'int32s' },
     0x8016 => { Name => 'SaturationAdj',        Writable => 'int32s' },
     0x8017 => { Name => 'ContrastAdj',          Writable => 'int32s' },
-    0x8018 => { Name => 'BrightnessAdj',        Writable => 'int32s' },
+    0x8018 => {
+        Name => 'BrightnessAdj',
+        Writable => 'int32s',
+        PrintConv => 'sprintf("%.2f", $val/300)', #JR
+        PrintConvInv => '$val * 300',
+    },
     0x8019 => { Name => 'HueAdj',               Writable => 'int32s' },
     0x801a => { Name => 'SharpnessAdj',         Writable => 'int32s' },
     0x801b => { Name => 'SharpnessOvershoot',   Writable => 'int32s' },
@@ -151,6 +157,15 @@ $VERSION = '1.02';
     0x8030 => { Name => 'PeripheralIllumCentralRadius', Writable => 'int32s' },
     0x8031 => { Name => 'PeripheralIllumCentralValue',  Writable => 'int32s' },
     0x8032 => { Name => 'PeripheralIllumPeriphValue',   Writable => 'int32s' },
+    0x8040 => { #JR
+        Name => 'DistortionCompensation',
+        Writable => 'int32s',
+        PrintConv => {
+            -1 => 'n/a', # (fixed by lens)
+            1 => 'On',
+            2 => 'Off',
+        },
+    },
     0x9000 => {
         Name => 'ToneCurveBrightnessX',
         Writable => 'int16u',
@@ -191,6 +206,52 @@ $VERSION = '1.02';
         Writable => 'int16u',
         Count => -1,
     },
+    0x900d => { #JR
+        Name => 'ChromaticAberrationCorrection', # "Magnification Chromatic Aberration"
+        Writable => 'int32s',
+        PrintConv => { 1 => 'On', 2 => 'Off' },
+    },
+    0x900e => { #JR
+        Name => 'InclinationCorrection',
+        Writable => 'int32u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    0x900f => { #JR
+        Name => 'InclinationAngle',
+        Writable => 'int32s',
+        PrintConv => 'sprintf("%.1f deg", $val/1000)',
+        PrintConvInv => 'ToFloat($val) * 1000',
+    },
+    0x9010 => { #JR
+        Name => 'Cropping',
+        Writable => 'int32u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    0x9011 => { #JR
+        Name => 'CropArea',
+        Writable => 'int32u',
+        Count => 4,
+    },
+    0x9012 => { #JR
+        Name => 'PreviewImageSize',
+        Writable => 'int32u',
+        Count => 2,
+    },
+    0x9013 => { #JR (ARQ images)
+        Name => 'PxShiftPeriphEdgeNR',
+        Writable => 'int32s',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    0x9014 => { #JR (ARQ images)
+        Name => 'PxShiftPeriphEdgeNRValue',
+        Writable => 'int32s',
+        PrintConv => 'sprintf("%.1f", $val/10)',
+        PrintConvInv => '$val * 10',
+    },
+    0x9017 => { Name => 'WhitesAdj',        Writable => 'int32s' }, #JR
+    0x9018 => { Name => 'BlacksAdj',        Writable => 'int32s' }, #JR
+    0x9019 => { Name => 'HighlightsAdj',    Writable => 'int32s' }, #JR
+    0x901a => { Name => 'ShadowsAdj',       Writable => 'int32s' }, #JR
     0xd000 => { Name => 'CurrentVersion',   Writable => 'int32u' },
     0xd001 => {
         Name => 'VersionIFD',
@@ -229,12 +290,14 @@ $VERSION = '1.02';
 %Image::ExifTool::SonyIDC::Composite = (
     GROUPS => { 2 => 'Image' },
     IDCPreviewImage => {
+        Groups => { 2 => 'Preview' },
         Require => {
             0 => 'IDCPreviewStart',
             1 => 'IDCPreviewLength',
         },
         # extract all preview images (not just one)
         RawConv => q{
+            @grps = $self->GetGroup($$val{0});
             require Image::ExifTool::SonyIDC;
             Image::ExifTool::SonyIDC::ExtractPreviews($self);
         },
@@ -259,35 +322,35 @@ Image::ExifTool::AddCompositeTags('Image::ExifTool::SonyIDC');
 #          or undef if there was no preview in the SonyIDC IFD
 sub ExtractPreviews($)
 {
-    my $exifTool = shift;
+    my $et = shift;
     my $i = 1;
     my $xtra = ' (1)';
     my $preview;
     # loop through all available IDC preview images in the order they were found
     for (;;) {
         my $key = "IDCPreviewStart$xtra";
-        unless (defined $$exifTool{VALUE}{$key}) {
+        unless (defined $$et{VALUE}{$key}) {
             last unless $xtra;
             $xtra = ''; # do the last tag extracted last
             next;
         }
         # run through IDC preview images in the same order they were extracted
-        my $off = $exifTool->GetValue($key) or last;
-        my $len = $exifTool->GetValue("IDCPreviewLength$xtra") or last;
+        my $off = $et->GetValue($key, 'ValueConv') or last;
+        my $len = $et->GetValue("IDCPreviewLength$xtra", 'ValueConv') or last;
         # get stack version from number in group 1 name
-        my $grp1 = $exifTool->GetGroup($key, 1);
+        my $grp1 = $et->GetGroup($key, 1);
         if ($grp1 =~ /(\d+)$/) {
             my $tag = "IDCPreviewImage$1";
             unless ($Image::ExifTool::Extra{$tag}) {
-                Image::ExifTool::AddTagToTable(\%Image::ExifTool::Extra, $tag, {
+                AddTagToTable(\%Image::ExifTool::Extra, $tag, {
                     Name => $tag,
-                    Groups => { 0 => 'Composite', 1 => 'Composite', 2 => 'Image'},
+                    Groups => { 0 => 'Composite', 1 => 'Composite', 2 => 'Preview'},
                 });
             }
-            my $val = Image::ExifTool::Exif::ExtractImage($exifTool, $off, $len, $tag);
-            $exifTool->FoundTag($tag, $val);
+            my $val = Image::ExifTool::Exif::ExtractImage($et, $off, $len, $tag);
+            $et->FoundTag($tag, $val, $et->GetGroup($key));
         } else {
-            $preview = Image::ExifTool::Exif::ExtractImage($exifTool, $off, $len, 'IDCPreviewImage');
+            $preview = Image::ExifTool::Exif::ExtractImage($et, $off, $len, 'IDCPreviewImage');
         }
         # step to next set of tags unless we are done
         last unless $xtra;
@@ -312,11 +375,11 @@ This module is used by Image::ExifTool
 =head1 DESCRIPTION
 
 This module contains definitions required by Image::ExifTool to read and
-write Sony Image Data Converter verson 3.0 metadata in ARW images.
+write Sony Image Data Converter version 3.0 metadata in ARW images.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
